
import grokcore.viewlet
import grokcore.view
from grokcore.view.meta.views import default_view_name
import grokcore.component
import martian

import zope.component
from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from zeam.form.composed.interfaces import ISubForm
from zeam.form.composed.form import SubFormBase


class SubFormGrokker(martian.ClassGrokker):
    """Grokker to register sub forms.
    """
    martian.component(SubFormBase)
    martian.directive(grokcore.component.context)
    martian.directive(grokcore.view.layer, default=IDefaultBrowserLayer)
    martian.directive(grokcore.viewlet.view)
    martian.directive(grokcore.view.name, get_default=default_view_name)

    def grok(self, name, factory, module_info, **kw):
        factory.module_info = module_info
        return super(SubFormGrokker, self).grok(
            name, factory, module_info, **kw)

    def execute(self, factory, config, context, layer, view, name, **kw):
        if not factory.prefix:
            factory.prefix = '%s.%s' % (view.prefix, name)

        adapts = (context, view, layer)
        config.action(
            discriminator=('adapter', adapts, ISubForm, name),
            callable=zope.component.provideAdapter,
            args=(factory, adapts, ISubForm, name),
            )
        return True




from megrok import pagetemplate as pt
from grokcore.viewlet.util import sort_components
from grokcore import component as grok
from zope import component

from zeam.form.base import form
from zeam.form.composed import interfaces

pt.templatedir('default_templates')


class SubFormBase(object):
    """Base class to be applied on a FormCanvas to get a subform.
    """
    grok.baseclass()

    # Set prefix to None, so it's changed by the grokker
    prefix = None

    def __init__(self, context, parent, request):
        super(SubFormBase, self).__init__(context, request)
        self.parent = parent

    def available(self):
        return True


class SubForm(SubFormBase, form.FormCanvas):
    """Form designed to be included in an another form (a
    ComposedForm).
    """
    grok.baseclass()
    grok.implements(interfaces.ISimpleSubForm)


class SubFormTemplate(pt.PageTemplate):
    """Default template for a SubForm
    """
    pt.view(SubForm)


class ComposedForm(form.Form):
    """A form which is composed of other forms (SubForm).
    """
    grok.baseclass()
    grok.implements(interfaces.IComposedForm)

    def __init__(self, context, request):
        super(ComposedForm, self).__init__(context, request)
        # retrieve subforms by adaptation
        subforms = map(lambda x: x[1], component.getAdapters(
                (self.context, self,  self.request), interfaces.ISubForm))
        # filter out unavailables ones
        subforms = filter(lambda x: x.available(), subforms)
        # sort them
        self.subforms = sort_components(subforms)

    def update(self):
        # Call update for all forms
        for subform in self.subforms:
            subform.update()

    def updateForm(self):
        # Set/run actions for all forms
        for subform in self.subforms:
            subform.updateActions()
        # Run our actions
        self.updateActions()
        # Set widgets for all forms
        for subform in self.subforms:
            subform.updateWidgets()


class ComposedFormTemplate(pt.PageTemplate):
    """Default template for a ComposedForm.
    """
    pt.view(ComposedForm)




from zope import interface

from zeam.form.base.interfaces import IForm, IFormCanvas, ISimpleFormCanvas
from zeam.form.base.interfaces import IZeamFormBaseAPI


class ISubForm(IFormCanvas):
    """A form designed to be included in an another form.
    """

    parent = interface.Attribute("Parent form")

    def available():
        """Return true if the form is available and should be
        rendered.
        """


class ISimpleSubForm(ISubForm, ISimpleFormCanvas):
    """A simple sub form.
    """


class IComposedForm(IForm):
    """A form which is composed of other forms.
    """

    subforms = interface.Attribute("List of subforms")


class IZeamFormComposedAPI(IZeamFormBaseAPI):
    """API exported by zeamf.form.composed.
    """

    ComposedForm = interface.Attribute(
        u"A form which can be compose of other forms")
    SubForm = interface.Attribute(
        u"A form included in a ComposedForm")

    view = interface.Attribute(
        u"Directive to select which ComposedForm a SubForm belongs to")
    order = interface.Attribute(
        u"Directive used to order SubForms between themselves")

